@@ -9,6 +9,11 @@
 #include "X11/Xaw/Label.h"
 #include <Xfwf/EzMenu.h>
 
+#ifdef SVR4                                     /* BB, 27 Mar '96 */
+#define index(s, c) strchr(s, c)
+#endif
+
+
 int Argc;
 char **Argv;
 
