@@ -128,6 +128,11 @@ static char rcsid[] = "$Id: FileComp.c,v 0.27 1993/04/20 10:49:56 thoth Exp thot
 #include <glob.h>
 #endif
 
+#ifdef SVR4                                     /* BB, 27 Mar '96 */
+#define bcopy(b1, b2, length) memcpy(b2, b1, length)
+#define bzero(b, length) memset(b, 0, length)
+#endif
+
 #define offset(field) XtOffset(XfwfFileCompleteWidget, xfwf_filecomp.field)
 static XtResource resources[] = {
   {XtNfilename, XtCFilename, XtRString, sizeof(String),
