@@ -67,6 +67,20 @@ static void activate(w, client_data, call_data)
 }
 
 
+/* ARGSUSED */
+static void print_stat(w, client_data, call_data)
+    Widget w;
+    XtPointer client_data, call_data;
+{
+    Bool expand = (Bool) call_data;
+
+    if (expand)
+        printf("Expanding %s...\n", XtName(w));
+    else
+        printf("Collapsing %s...\n", XtName(w));
+}
+
+
 /* ARGSUSED */
 static void quit(w, client_data, call_data)
     Widget w;
@@ -107,6 +121,9 @@ void main(argc, argv)
     label3 = XtVaCreateManagedWidget
 	("l3", xfwfLabelWidgetClass, tree3, NULL);
     XtRealizeWidget(toplevel);
+    XtAddCallback(tree1, XtNactivate, print_stat, NULL);
+    XtAddCallback(tree2, XtNactivate, print_stat, NULL);
+    XtAddCallback(tree3, XtNactivate, print_stat, NULL);
     XtAddCallback(button1, XtNactivate, activate, NULL);
     XtAddCallback(button2, XtNactivate, activate, NULL);
     XtAddCallback(button3, XtNactivate, quit, NULL);
