@@ -23,8 +23,9 @@
  |  Bert Bos <bert@let.rug.nl> (31 Oct 1994)
  +--------------------------------------------------------------------*/
 
-#define USE_PTY					/* Use ptys or pipes? */
+#define USE_PTY                                 /* Use ptys or pipes? */
 /* #define USE_CLONE				/* Use /dev/ptym/clone? */
+/* #define USE_PTYIO_H                          /* Use sys/ptyio.h file? */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -34,8 +35,12 @@
 #include <fcntl.h>
 #include <sys/wait.h>
 #ifdef USE_PTY
+#ifdef USE_PTYIO_H
 #include <sys/ptyio.h>
-#endif
+#else /* USE_PTYIO_H */
+#include <sys/termios.h>
+#endif /* USE_PTYIO_H */
+#endif /* USE_PTY */
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <Xfwf/XmATerm.h>
@@ -229,9 +234,10 @@ void main(argc, argv)
     Widget toplevel, ansiterm;
     pid_t child;
 #ifndef USE_PTY
-    int pfdout[2], pfdin[2];
+    int pfdout[2], pfdin[2];                    /* Use two pipes */
 #else
-    int master, slave;
+    int master, slave;                          /* Use a pty */
+    static int one = 1;                         /* For ioctl() argument */
 #endif
 
     toplevel = XtVaAppInitialize(&app_context, "AnsiTermT", NULL, 0, &argc,
@@ -252,8 +258,8 @@ void main(argc, argv)
 #else
     if (! openpty(&master, &slave)) syserr("pty");
 #if 1
-    /* Enable flow control */
-    if (ioctl(master, TIOCREMOTE, TRUE) == -1) syserr("ioctl");
+    /* Enable flow control (I'm still not sure what this does...) */
+    if (ioctl(master, TIOCREMOTE, &one) == -1) syserr("ioctl");
 #endif
 #endif
     signal(SIGCHLD, finish);			/* Catch dying children */
@@ -284,7 +290,7 @@ void main(argc, argv)
 	if (close(2) == -1) syserr("close");
 	if (dup(slave) != 2) syserr("dup failed\n");
 #endif
-	{
+	{                                       /* Close all other files */
 	    long c, tblsiz = sysconf(_SC_OPEN_MAX);
 	    for (c = 3; c < tblsiz; c++) (void) close(c);
 	}
@@ -292,7 +298,7 @@ void main(argc, argv)
 #if 0
 	execlp("telnet", "telnet", "www.cern.ch", NULL);
 #else
-	execlp("ksh", "-ksh", NULL);
+	execlp("sh", "-sh", NULL);
 #endif
 	syserr("execlp");
 	/* NOTREACHED */
