@@ -5,15 +5,19 @@
 #include <Xfwf/XmTabs.h>
 
 #define TABHT 30
+#define NR_OF_LABELS 7
 
 static String fallback_resources[] = {
     "XmTabsT.geometry:		600x600",
-    "*board*background:		#bdbdbdbdbdbd",
+    "*board*background:		#0092aa",
+    "*board.background:		#55b6aa",
     "*board.shadowThickness:	0",
     "*board.borderWidth:	0",
     "*XmTabs.shadowThickness:	2",
     "*XmTabs.font:		*-helvetica-medium-r-*--14-*",
-    "*XmTabs.tabcolor:		#a5a5a5a5a5a5",
+    "*XmTabs.tabcolor:		#5592aa",
+    "*XmTabs.labels:\
+|Introduction|Clicks||About the `Quit' tab|More filler|Pixel trick|Quit",
     /*
      * tabs along the top:
      * tab is as wide as label, plus default textmargin
@@ -74,16 +78,6 @@ connected.",
     "This won't get displayed",
 };
 
-static String labels[] = {
-    "Introduction",
-    "Clicks",
-    "",						/* An empty label */
-    "About the `Quit' tab",
-    "More filler",
-    "Pixel trick",
-    "Quit",
-};
-
 static int cur = 0;				/* Current label/text */
 
 
@@ -97,24 +91,24 @@ static void handle_click(w, client_data, call_data)
     if (tab == 0) return;
     cur += tab;
 
-    if (cur == XtNumber(labels) - 1) exit(0);	/* Quit button? */
+    if (cur == NR_OF_LABELS - 1) exit(0);	/* Quit button? */
 
     /* Change tabs */
     XtVaSetValues(tabs1,
 		  XtNlefttabs, cur,
-		  XtNrighttabs, XtNumber(labels) - cur - 1,
+		  XtNrighttabs, NR_OF_LABELS - cur - 1,
 		  NULL);
     XtVaSetValues(tabs2,
 		  XtNlefttabs, cur,
-		  XtNrighttabs, XtNumber(labels) - cur - 1,
+		  XtNrighttabs, NR_OF_LABELS - cur - 1,
 		  NULL);
     XtVaSetValues(tabs3,
 		  XtNlefttabs, cur,
-		  XtNrighttabs, XtNumber(labels) - cur - 1,
+		  XtNrighttabs, NR_OF_LABELS - cur - 1,
 		  NULL);
     XtVaSetValues(tabs4,
 		  XtNlefttabs, cur,
-		  XtNrighttabs, XtNumber(labels) - cur - 1,
+		  XtNrighttabs, NR_OF_LABELS - cur - 1,
 		  NULL);
     /* Change text */
     XtVaSetValues(label, XtVaTypedArg, XmNlabelString,
@@ -132,9 +126,8 @@ void main(argc, argv)
 	("board", xmFormWidgetClass, toplevel, NULL);
     tabs1 = XtVaCreateManagedWidget
 	("tabs1", xmTabsWidgetClass, board,
-	 XtNlabels, labels,
 	 XtNlefttabs, cur,
-	 XtNrighttabs, XtNumber(labels) - 1 - cur,
+	 XtNrighttabs, NR_OF_LABELS - 1 - cur,
 	 XtNorientation, XfwfUpTabs,
 	 XmNtopAttachment, XmATTACH_FORM,
 	 XmNleftAttachment, XmATTACH_FORM, XmNleftOffset, TABHT,
@@ -147,9 +140,8 @@ void main(argc, argv)
     
     tabs2 = XtVaCreateManagedWidget
 	("tabs2", xmTabsWidgetClass, board,
-	 XtNlabels, labels,
 	 XtNlefttabs, cur,
-	 XtNrighttabs, XtNumber(labels) - 1 - cur,
+	 XtNrighttabs, NR_OF_LABELS - 1 - cur,
 	 XtNorientation, XfwfLeftTabs,
 	 XmNleftAttachment, XmATTACH_FORM,
 	 XmNtopAttachment, XmATTACH_FORM, XmNtopOffset, TABHT,
@@ -158,9 +150,8 @@ void main(argc, argv)
 	 NULL);
     tabs3 = XtVaCreateManagedWidget
 	("tabs3", xmTabsWidgetClass, board,
-	 XtNlabels, labels,
 	 XtNlefttabs, cur,
-	 XtNrighttabs, XtNumber(labels) - 1 - cur,
+	 XtNrighttabs, NR_OF_LABELS - 1 - cur,
 	 XtNorientation, XfwfDownTabs,
 	 XmNbottomAttachment, XmATTACH_FORM,
 	 XmNleftAttachment, XmATTACH_FORM, XmNleftOffset, TABHT,
@@ -169,9 +160,8 @@ void main(argc, argv)
 	 NULL);
     tabs4 = XtVaCreateManagedWidget
 	("tabs4", xmTabsWidgetClass, board,
-	 XtNlabels, labels,
 	 XtNlefttabs, cur,
-	 XtNrighttabs, XtNumber(labels) - 1 - cur,
+	 XtNrighttabs, NR_OF_LABELS - 1 - cur,
 	 XtNorientation, XfwfRightTabs,
 	 XmNrightAttachment, XmATTACH_FORM,
 	 XmNtopAttachment, XmATTACH_FORM, XmNtopOffset, TABHT,
