@@ -1,4 +1,4 @@
-/* $Id: XCC.c,v 1.3 1995/01/31 23:03:10 cwikla Exp $ */
+/* $Id: XCC.c,v 1.18 1995/10/27 19:19:45 cwikla Exp $ */
 /*
  * Copyright 1994,1995 John L. Cwikla
  *
@@ -39,6 +39,42 @@
 
 #define NUMBER(a) ((int)(sizeof(a)/sizeof(a[0])))
 
+#if NeedFunctionProtoTypes
+static int _pixelSort(const void *_arg1, const void *_arg2)
+#else
+static int _pixelSort(_arg1, _arg2)
+void *_arg1;
+void *_arg2;
+#endif /* NeedFunctionProtoTypes */
+{
+	return ( ((XColor *)_arg1)->pixel - ((XColor *)_arg2)->pixel);
+}
+
+#if NeedFunctionProtoTypes
+static void _queryColors(XCC _xcc)
+#else
+static void _queryColors(_xcc)
+XCC _xcc;
+#endif
+{
+	int i;
+
+	_xcc->CMAP = (XColor *)Xmalloc(sizeof(XColor) * _xcc->numColors);
+	if (_xcc->CMAP)
+	{
+		for(i=0;i<_xcc->numColors;i++)
+			_xcc->CMAP[i].pixel = _xcc->CLUT ? _xcc->CLUT[i] : _xcc->stdCmap.base_pixel + i;
+
+		if (!XQueryColors(_xcc->dpy, _xcc->colormap, _xcc->CMAP, _xcc->numColors))
+		{
+			XFree(_xcc->CMAP);
+			_xcc->CMAP = NULL;
+		}
+		else
+			qsort(_xcc->CMAP, _xcc->numColors, sizeof(XColor), _pixelSort);
+	}
+}
+
 /*
 ** Find a standard colormap from a property, and make sure the visual matches
 ** the one we are using!
@@ -61,6 +97,7 @@ XStandardColormap *_matchedCube;
 	if (!_atom)
 		return 0;
 
+	cubes = (XStandardColormap *)NULL;
 	status = XGetRGBColormaps(_xcc->dpy, RootWindow(_xcc->dpy, DefaultScreen(_xcc->dpy)),
 				&cubes, &count, _atom);
 
@@ -142,11 +179,39 @@ Atom _atom;
 	if (!status)
 		return 0;
 
+/*
+** This is a hack to force standard colormaps that don't set green/blue max
+** to work correctly.  For instance RGB_DEFAULT_GRAY has these set if xstdcmap
+** is used, but not if xscm was. Plus this also makes RGB_RED_MAP (etc) work.
+*/
 
-	_xcc->mode = MODE_STDCMAP;
-	_xcc->numColors = _xcc->stdCmap.red_max * _xcc->stdCmap.red_mult + 
-					 _xcc->stdCmap.green_max * _xcc->stdCmap.green_mult +
-					 _xcc->stdCmap.blue_max * _xcc->stdCmap.blue_mult;
+	_xcc->colormap = _xcc->stdCmap.colormap;
+
+	if ( (!_xcc->stdCmap.green_max + !_xcc->stdCmap.blue_max + !_xcc->stdCmap.red_max) > 1)
+	{
+		_xcc->mode = MODE_MY_GRAY;
+		if (_xcc->stdCmap.green_max)
+		{
+			_xcc->stdCmap.red_max = _xcc->stdCmap.green_max;
+			_xcc->stdCmap.red_mult = _xcc->stdCmap.green_mult;
+		}
+		else
+		if (_xcc->stdCmap.blue_max)
+		{
+			_xcc->stdCmap.red_max = _xcc->stdCmap.blue_max;
+			_xcc->stdCmap.red_mult = _xcc->stdCmap.blue_mult;
+		}
+		_xcc->stdCmap.green_max = _xcc->stdCmap.blue_max = 0;
+		_xcc->stdCmap.green_mult = _xcc->stdCmap.blue_mult = 0;
+	}
+	else
+		_xcc->mode = MODE_STDCMAP;
+
+	_xcc->numColors = (_xcc->stdCmap.red_max+1) *
+					 (_xcc->stdCmap.green_max+1) *
+					 (_xcc->stdCmap.blue_max+1);
+
+	_queryColors(_xcc);
 
 	return status;
 }
@@ -204,7 +269,7 @@ XCC _xcc;
 
 retryGray:
 
-	dinc = 65535.0/_xcc->numColors;
+	dinc = 65535.0/(_xcc->numColors-1);
 
 	clrs = cstart;
 	for(i=0;i<_xcc->numColors;i++)
@@ -234,12 +299,14 @@ retryGray:
 
 	_xcc->stdCmap.colormap = _xcc->colormap;
 	_xcc->stdCmap.base_pixel = 0;
-	_xcc->stdCmap.red_max = _xcc->visual->map_entries-1;
+	_xcc->stdCmap.red_max = _xcc->numColors-1;
 	_xcc->stdCmap.green_max = 0;
 	_xcc->stdCmap.blue_max = 0;
 	_xcc->stdCmap.red_mult = 1;
 	_xcc->stdCmap.green_mult = _xcc->stdCmap.blue_mult = 0;
 
+	_queryColors(_xcc);
+
 	_xcc->mode = MODE_MY_GRAY;
 }
 
@@ -287,9 +354,18 @@ retryColor:
 		{
 			for(blue = 0; blue <= _xcc->stdCmap.blue_max; blue++)
 			{
-				clrs->red = 0xFFFF * red / _xcc->stdCmap.red_max;
-				clrs->green = 0xFFFF * green / _xcc->stdCmap.green_max;
-				clrs->blue = 0xFFFF * blue / _xcc->stdCmap.blue_max;
+				if (_xcc->stdCmap.red_max)
+					clrs->red = 0xFFFF * red / _xcc->stdCmap.red_max;
+				else
+					clrs->red = 0;
+				if (_xcc->stdCmap.green_max)
+					clrs->green = 0xFFFF * green / _xcc->stdCmap.green_max;
+				else
+					clrs->green = 0;
+				if (_xcc->stdCmap.blue_max)
+					clrs->blue = 0xFFFF * blue / _xcc->stdCmap.blue_max;
+				else
+					clrs->blue = 0;
 
 				if (!XAllocColor(_xcc->dpy, _xcc->colormap, clrs))
 				{
@@ -316,6 +392,8 @@ retryColor:
 
 	_xcc->numColors = count;
 
+	_queryColors(_xcc);
+
 	XFree((char *)cstart);
 
 	_xcc->mode = MODE_STDCMAP;
@@ -385,7 +463,7 @@ XCC _xcc;
 
 /*
 ** Cheat here! Make the direct color visual work like
-** a true color! I'm pretty sure this is right!
+** a true color! USE the CLUT!!!
 */
 
 #if NeedFunctionProtoTypes
@@ -397,7 +475,7 @@ XCC _xcc;
 {
 	int n, count;
 	XColor *clrs, *cstart;
-	unsigned long rval, bval, gval, maxval;
+	unsigned long rval, bval, gval;
 	unsigned long *rtable;
 	unsigned long *gtable;
 	unsigned long *btable;
@@ -405,45 +483,38 @@ XCC _xcc;
 
 	_initTrueColor(_xcc); /* for shift stuff */
 
-	rval = _xcc->visualInfo->red_mask;
-	gval = _xcc->visualInfo->green_mask;
-	bval = _xcc->visualInfo->blue_mask;
-
-	while(!(rval & 1))
-		rval >>= 1;
-	while(!(gval & 1))
-		gval >>= 1;
-	while(!(bval & 1))
-		bval >>= 1;
+	rval = _xcc->visualInfo->red_mask >> _xcc->shifts.red;
+	gval = _xcc->visualInfo->green_mask >> _xcc->shifts.green;
+	bval = _xcc->visualInfo->blue_mask >> _xcc->shifts.blue;
 
 	rtable = (unsigned long *)Xmalloc(sizeof(unsigned long) * (rval+1));
 	gtable = (unsigned long *)Xmalloc(sizeof(unsigned long) * (gval+1)); 
 	btable = (unsigned long *)Xmalloc(sizeof(unsigned long) * (bval+1));
 
-	maxval = (rval > gval) ? rval : gval;
-	maxval = (maxval > bval) ? maxval : bval;
+	_xcc->maxEntry = (rval > gval) ? rval : gval;
+	_xcc->maxEntry = (_xcc->maxEntry > bval) ? _xcc->maxEntry : bval;
 
-	cstart = (XColor *)Xmalloc(sizeof(XColor) * (maxval+1));
-	_xcc->CLUT = (unsigned long *)Xmalloc(sizeof(unsigned long) * (maxval+1));
+	cstart = (XColor *)Xmalloc(sizeof(XColor) * (_xcc->maxEntry+1));
+	_xcc->CLUT = (unsigned long *)Xmalloc(sizeof(unsigned long) * (_xcc->maxEntry+1));
 
 retrydirect:
 
 	for(n=0;n<=rval;n++)
-		rtable[n] = 65535.0/rval * n;
+		rtable[n] = rval ? 65535.0/(double)rval * n : 0;
 	for(n=0;n<=gval;n++)
-		gtable[n] = 65535.0/gval * n;
+		gtable[n] = gval ? 65535.0/(double)gval * n : 0;
 	for(n=0;n<=bval;n++)
-		btable[n] = 65535.0/bval * n;
+		btable[n] = bval ? 65535.0/bval * n : 0;
 
-	maxval = (rval > gval) ? rval : gval;
-	maxval = (maxval > bval) ? maxval : bval;
+	_xcc->maxEntry = (rval > gval) ? rval : gval;
+	_xcc->maxEntry = (_xcc->maxEntry > bval) ? _xcc->maxEntry : bval;
 
 	count = 0;
 	clrs = cstart;
 	_xcc->numColors = (bval + 1) * (gval + 1) * (rval + 1);
-	for(n=0;n<=maxval;n++)
+	for(n=0;n<=_xcc->maxEntry;n++)
 	{
-		dinc = (double)n/(double)maxval;
+		dinc = (double)n/(double)_xcc->maxEntry;
 		clrs->red = rtable[(int)(dinc * rval)];
 		clrs->green = gtable[(int)(dinc * gval)];
 		clrs->blue = btable[(int)(dinc * bval)];
@@ -492,6 +563,41 @@ retrydirect:
 	XFree(cstart);
 }
 
+#if NeedFunctionProtoTypes
+XCC XCCMonoCreate(Display *_dpy, Visual *_visual, Colormap _colormap)
+#else
+XCC XCCMonoCreate(_dpy, _visual, _colormap)
+Display *_dpy;
+Visual *_visual;
+Colormap _colormap;
+#endif /* NeedFunctionProtoTypes */
+{
+	XCC xcc;
+	XVisualInfo visInfo;
+	int n;
+
+	xcc = (XCC)Xmalloc(sizeof(struct _XColorContext));
+   
+	if (xcc == NULL)
+		return NULL;
+
+	xcc->dpy = _dpy;
+	xcc->visual = _visual;
+	xcc->colormap = _colormap;
+	xcc->CLUT = NULL;
+	xcc->CMAP = NULL;
+	xcc->mode = MODE_UNDEFINED;
+	xcc->needToFreeColormap = 0;
+
+	visInfo.visualid = XVisualIDFromVisual(_visual);
+	xcc->visualInfo = XGetVisualInfo(_dpy, VisualIDMask, &visInfo, &n);
+
+	_initBW(xcc);
+
+	return xcc;
+}
+
+
 #if NeedFunctionProtoTypes
 XCC XCCCreate(Display *_dpy, Visual *_visual, int _usePrivateColormap, int _useStdCmaps, 
 							Atom _stdCmapAtom, Colormap *_colormap)
@@ -505,68 +611,72 @@ Atom _stdCmapAtom;
 Colormap *_colormap;
 #endif /* NeedFunctionProtoTypes */
 {
-	XCC cc;
+	XCC xcc;
 	int n;
 	XVisualInfo visInfo;
 	int status;
 	int retryCount;
 
-	cc = (XCC)Xmalloc(sizeof(struct _XColorContext));
+	xcc = (XCC)Xmalloc(sizeof(struct _XColorContext));
 
-	if (cc == NULL)
+	if (xcc == NULL)
 		return NULL;
 
-	cc->dpy = _dpy;
-	cc->visual = _visual;
-	cc->colormap = DefaultColormap(_dpy, DefaultScreen(_dpy));
-	cc->CLUT = NULL;
-	cc->mode = MODE_UNDEFINED;
-	cc->needToFreeColormap = 0;
+	xcc->dpy = _dpy;
+	xcc->visual = _visual;
+	xcc->colormap = DefaultColormap(_dpy, DefaultScreen(_dpy));
+	xcc->CLUT = NULL;
+	xcc->CMAP = NULL;
+	xcc->mode = MODE_UNDEFINED;
+	xcc->needToFreeColormap = 0;
 
 	visInfo.visualid = XVisualIDFromVisual(_visual);
-	cc->visualInfo = XGetVisualInfo(_dpy, VisualIDMask, &visInfo, &n);
+	xcc->visualInfo = XGetVisualInfo(_dpy, VisualIDMask, &visInfo, &n);
 
 	status = 0;
 	if (!_usePrivateColormap && _useStdCmaps)
-		status = _lookForStdCmap(cc, _stdCmapAtom);
+		status = _lookForStdCmap(xcc, _stdCmapAtom);
 
 	retryCount = 0;
 	while(retryCount < 2)
 	{
 		if (!status)
 		{
-			if (_usePrivateColormap || (cc->visual != DefaultVisual(_dpy, DefaultScreen(_dpy))))
+			if (_usePrivateColormap || (xcc->visual != DefaultVisual(_dpy, DefaultScreen(_dpy))))
 			{
-				cc->colormap = XCreateColormap(_dpy, RootWindow(_dpy, DefaultScreen(_dpy)), 
-								cc->visual, AllocNone);
-				cc->needToFreeColormap = (cc->colormap != DefaultColormap(_dpy, DefaultScreen(_dpy)));
+				xcc->colormap = XCreateColormap(_dpy, RootWindow(_dpy, DefaultScreen(_dpy)), 
+								xcc->visual, AllocNone);
+				xcc->needToFreeColormap = (xcc->colormap != DefaultColormap(_dpy, DefaultScreen(_dpy)));
 			
 			}
 			switch(_visual->class)
 			{
 				case StaticGray:
 				case GrayScale:
-					_initGray(cc);
+					if (xcc->visual->map_entries == 2)
+						_initBW(xcc);
+					else
+						_initGray(xcc);
 					break;
 
 				case TrueColor: /* shifts */
-					_initTrueColor(cc);
+					_initTrueColor(xcc);
 					break;
 			
 				case DirectColor: /* shifts & fake CLUT */
-					_initDirectColor(cc);
+					_initDirectColor(xcc);
 					break;
 
 				case StaticColor:
 				case PseudoColor:
-					_initColor(cc);
+					_initColor(xcc);
 					break;
 			}
 		}
 		else
-			cc->colormap = cc->stdCmap.colormap;
+			xcc->colormap = xcc->stdCmap.colormap;
 
-		if ( (cc->mode == MODE_BW) && (cc->visualInfo->depth > 1))
+		if ( (xcc->mode == MODE_BW) && (xcc->visualInfo->depth > 1))
 		{
 			_usePrivateColormap = 1;
 			retryCount++;
@@ -575,9 +685,9 @@ Colormap *_colormap;
 			break;
 	}
 
-	*_colormap = cc->colormap;
+	*_colormap = xcc->colormap;
 
-	return cc;
+	return xcc;
 }
 
 /*
@@ -600,9 +710,13 @@ XCC _xcc;
 		XFree((char *)_xcc->CLUT);
 	}
 
+	if (_xcc->CMAP != NULL)
+		XFree((char *)_xcc->CMAP);
+
 	if (_xcc->needToFreeColormap)
 		XFreeColormap(_xcc->dpy, _xcc->colormap);
 
+	XFree(_xcc->visualInfo);
 	XFree((char *)_xcc);
 }
 
@@ -633,6 +747,8 @@ unsigned int _blue;
 		}
 		case MODE_MY_GRAY:
 				_red = _red * 0.3 + _green * 0.59 + _blue * 0.1;
+				_green = 0;
+				_blue = 0;
 				/* fall thru */
 
 		case MODE_STDCMAP:
@@ -657,19 +773,24 @@ unsigned int _blue;
 				return _xcc->stdCmap.base_pixel + ired + igreen + iblue;
 
 		case MODE_TRUE:
-			_red >>= 16 - _xcc->bits.red;
-			_green >>= 16 - _xcc->bits.green;
-			_blue >>= 16 - _xcc->bits.blue;
+			if (_xcc->CLUT == NULL)
+			{
+				_red >>= 16 - _xcc->bits.red;
+				_green >>= 16 - _xcc->bits.green;
+				_blue >>= 16 - _xcc->bits.blue;
 
-			ired = _red << _xcc->shifts.red;
+				ired = (_red << _xcc->shifts.red) & _xcc->masks.red;
+				igreen = (_green << _xcc->shifts.green) & _xcc->masks.green;
+				iblue = (_blue << _xcc->shifts.blue) & _xcc->masks.blue;
+			}
+			else
+			{
+				ired = _xcc->CLUT[(int)((_red * _xcc->maxEntry)/65535)] & _xcc->masks.red;
+				igreen = _xcc->CLUT[(int)((_green * _xcc->maxEntry)/65535)] & _xcc->masks.green;
+				iblue = _xcc->CLUT[(int)((_blue * _xcc->maxEntry)/65535)] & _xcc->masks.blue;
+			}
 
-			igreen = _green << _xcc->shifts.green;
-
-			iblue = _blue << _xcc->shifts.blue;
-
-			return (ired & _xcc->masks.red) +
-					(igreen & _xcc->masks.green) +
-					(iblue & _xcc->masks.blue);
+			return ired | igreen | iblue;
 
 		default:
 			color.red = _red;
@@ -692,3 +813,153 @@ XCC _xcc;
 {
 	return _xcc->numColors;
 }
+
+#if NeedFunctionProtoTypes
+Colormap XCCGetColormap(XCC _xcc)
+#else
+Colormap XCCGetColormap(_xcc)
+XCC _xcc;
+#endif /* NeedFunctionProtoTypes */
+{
+	if (_xcc)
+		return _xcc->colormap;
+	else
+		return (Colormap)0;
+}
+
+#if NeedFunctionProtoTypes
+Visual *XCCGetVisual(XCC _xcc)
+#else
+Visual *XCCGetVisual(_xcc)
+XCC _xcc;
+#endif /* NeedFunctionProtoTypes */
+{
+	if (_xcc)
+		return _xcc->visual;
+	else
+		return (Visual *)NULL;
+}
+
+#if NeedFunctionProtoTypes
+XVisualInfo *XCCGetVisualInfo(XCC _xcc)
+#else
+XVisualInfo *XCCGetVisualInfo(_xcc)
+XCC _xcc;
+#endif /* NeedFunctionProtoTypes */
+{
+	if (_xcc)
+		return _xcc->visualInfo;
+	else
+		return (XVisualInfo *)NULL;
+}
+
+#if NeedFunctionProtoTypes
+int XCCQueryColors(XCC _xcc, XColor *_colors, int _numColors)
+#else
+int XCCQueryColors(_xcc, _colors, _numColors)
+XCC _xcc;
+XColor *_colors;
+int _numColors;
+#endif /* NeedFunctionProtoTypes */
+{
+	int i;
+	XColor *tc, *curTC;
+
+	switch(_xcc->mode)
+	{
+		case MODE_BW:
+			for(i=0,tc=_colors;i<_numColors;i++,tc++)
+			{
+				if (tc->pixel == _xcc->whitePixel)
+					tc->red = tc->green = tc->blue = 65535;
+				else
+					tc->red = tc->green = tc->blue = 0;
+			}
+			break;
+
+		case MODE_TRUE:
+			if (_xcc->CLUT == NULL)
+			{
+				for(i=0,tc=_colors;i<_numColors;i++,tc++)
+				{
+					tc->red = ((tc->pixel & _xcc->masks.red) * 65535)/_xcc->masks.red;
+					tc->green = ((tc->pixel & _xcc->masks.green) * 65535)/_xcc->masks.green;
+					tc->blue = ((tc->pixel & _xcc->masks.blue) * 65535)/_xcc->masks.blue;
+				}
+			}
+			else
+				return XQueryColors(_xcc->dpy, _xcc->colormap, _colors, _numColors);
+			break;
+
+		case MODE_STDCMAP:
+		default:
+			if (_xcc->CMAP == NULL)
+				return XQueryColors(_xcc->dpy, _xcc->colormap, _colors, _numColors);
+			else
+			{
+				register int first, last, half;
+				unsigned long halfPixel;
+
+				for(i=0,tc=_colors;i<_numColors;i++)
+				{
+					first = 0;
+					last = _xcc->numColors-1;
+					curTC = tc;
+
+					while(first <= last)
+					{
+						half = (first+last)/2;
+						halfPixel = _xcc->CMAP[half].pixel;
+						if (tc->pixel == halfPixel)
+						{
+							tc->red = _xcc->CMAP[half].red;
+							tc->green = _xcc->CMAP[half].green;
+							tc->blue = _xcc->CMAP[half].blue;
+							tc++;
+							first = last+1; /* fake break */
+						}
+						else
+						if (tc->pixel > halfPixel)
+							first = half+1;
+						else
+							last = half-1;
+					}
+					/* if have to get one, might as well get them all! */
+					if (tc == curTC)
+						return XQueryColors(_xcc->dpy, _xcc->colormap, _colors, _numColors);
+				}
+
+				return 1;
+			}
+			break;
+		}
+
+	return 1;
+}
+
+#if NeedFunctionProtoTypes
+int XCCQueryColor(XCC _xcc, XColor *_color)
+#else
+int XCCQueryColor(_xcc, _color)
+XCC _xcc;
+XColor *_color;
+#endif /* NeedFunctionProtoTypes */
+{
+	return XCCQueryColors(_xcc, _color, 1);
+}
+
+
+#if NeedFunctionProtoTypes
+Display *XCCGetDisplay(XCC _xcc)
+#else
+Display *XCCGetDisplay(_xcc)
+XCC _xcc;
+#endif /* NeedFunctionProtoTypes */
+{
+	if (_xcc)
+		return _xcc->dpy;
+	else
+		return (Display *)NULL;
+}
+
+
